/*
 * GQradio
 * (C) 2002 John Ellis
 *
 * Author: John Ellis
 *
 * This software is released under the GNU General Public License (GNU GPL).
 * Please read the included file COPYING for more information.
 * This software comes with no warranty of any kind, use at your own risk!
 */


#include "gqradio.h"
#include "preferences.h"

#include "display.h"
#include "io_radio.h"
#include "mixer.h"
#include "preset.h"
#include "ui2_main.h"
#include "ui_clist_edit.h"
#include "ui_fileops.h"
#include "ui_tabcomp.h"
#include "ui_utildlg.h"

static GtkWidget *configwindow;
static GtkWidget *mixer_entry;
static GtkWidget *default_skin_entry;
static GtkWidget *preset_clist;
static GtkWidget *device_entry;

static gint c_slik_smart_placement;
static gint c_slik_remember_position;
static gint c_mixer_device_id;
static gint c_wm_decorations;
static gint c_slik_double_size;
static gint c_description_scrolls_always;
static gint c_skinned_menus_enable;

static gint c_slik_colorshift_on;
static gint c_slik_colorshift_r;
static gint c_slik_colorshift_g;
static gint c_slik_colorshift_b;
static gint c_slik_colorshift_a;

static gint c_slik_transparency_force;
static gint c_slik_transparency_force_a;

static gint c_use_reported_freq_limits;

static gint c_freq_step;


static void create_config_window(gint start_tab);


static void hscale_cb(GtkObject *adj, gpointer data)
{
	gint *c_n = data;

	*c_n = (gint)(GTK_ADJUSTMENT(adj)->value / 100.0 * 255.0);
}

static GtkWidget *hscale_new(gint n, gint *c_n)
{
	GtkObject *adj;
        GtkWidget *hscale;

	*c_n = n;
	adj = gtk_adjustment_new((float)n / 255.0 * 100.0, 0.0, 100.0, 1.0, 1.0, 0.0);
        hscale = gtk_hscale_new (GTK_ADJUSTMENT(adj));
        gtk_range_set_update_policy (GTK_RANGE (hscale), GTK_UPDATE_DELAYED);
        gtk_scale_set_digits (GTK_SCALE (hscale), 0);
        gtk_signal_connect( GTK_OBJECT(adj),"value_changed", GTK_SIGNAL_FUNC(hscale_cb), c_n);

	return hscale;
}

static void checkbox_cb(GtkWidget *w, gpointer data)
{
	gint *c_n = data;

	*c_n = GTK_TOGGLE_BUTTON (w)->active;
}

static GtkWidget *checkbox_new(gchar *text, gint n, gint *c_n)
{
        GtkWidget *button;

	*c_n = n;
	button = gtk_check_button_new_with_label(text);
	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(button), n);
	gtk_signal_connect (GTK_OBJECT(button), "clicked", (GtkSignalFunc)checkbox_cb, c_n);

	return button;
}

static gint preset_clist_frequency_cb(ClistEditData *ced,
				      const gchar *old_text, const gchar *new_text, gpointer data)
{
	guint32 freq;
	gint ret = TRUE;
	const gchar *path;
	gchar *new_path;

	freq = radio_freq_from_text(new_text);

	path = gtk_clist_get_row_data(GTK_CLIST(ced->clist), ced->row);
	if (freq < radio_limit_get_lower() || freq > radio_limit_get_upper())
		{
		gtk_clist_set_text(GTK_CLIST(ced->clist), ced->row, 1, "");
		gtk_clist_set_text(GTK_CLIST(ced->clist), ced->row, 2, "");
		if (!path) return FALSE;
		new_path = g_strdup("clear");
		ret = FALSE;
		}
	else
		{
		gchar *tmp;
		if (path)
			{
			tmp = radio_path_to_title(path);
			}
		else
			{
			tmp = g_strdup(preset_get_description(ced->row));
			}
		new_path = radio_freq_to_path(freq, tmp);
		g_free(tmp);
		}

	gtk_clist_set_row_data_full(GTK_CLIST(ced->clist), ced->row,
				    new_path, (GtkDestroyNotify)g_free);

	return ret;
}

static gint preset_clist_description_cb(ClistEditData *ced,
					const gchar *old_text, const gchar *new_text, gpointer data)
{
	const gchar *path;
	gchar *new_path;

	path = gtk_clist_get_row_data(GTK_CLIST(ced->clist), ced->row);
	if (path)
		{
		new_path = radio_freq_to_path(radio_path_to_freq(path), new_text);
		}
	else
		{
		new_path = radio_freq_to_path(preset_get_freq(ced->row), new_text);
		}

	gtk_clist_set_row_data_full(GTK_CLIST(ced->clist), ced->row,
				    new_path, (GtkDestroyNotify)g_free);

	return TRUE;
}

static void preset_clist_click_cb(GtkWidget *clist, GdkEventButton *event, gpointer data)
{
	gint row = -1;
	gint column = -1;

	gtk_clist_get_selection_info(GTK_CLIST(clist), event->x, event->y, &row, &column);
	if (row == -1 || column == -1) return;

	if (column == 1)
		{
		if (event->button == 1)
			{
			clist_edit_by_row(GTK_CLIST(clist), row, column,
                                          preset_clist_frequency_cb, NULL);
			}
		else if (event->button == 2 || event->button == 3 ||
			 event->button == 4 || event->button == 5)
			{
			guint32 freq;
			const gchar *path;
			gchar *new_path;
			gchar *tmp;

			path = gtk_clist_get_row_data(GTK_CLIST(clist), row);
			freq = (path) ? radio_path_to_freq(path) : preset_get_freq(row);

			if (event->button == 4 || event->button == 3)
				{
				freq += RADIO_SCAN_INCREMENT;
				}
			else
				{
				freq -= RADIO_SCAN_INCREMENT;
				}
			freq = radio_freq_clamp_to_increment(freq, RADIO_SCAN_INCREMENT);
			CLAMP(freq, radio_limit_get_lower(), radio_limit_get_upper());

			tmp = (path) ? radio_path_to_title(path) : g_strdup(preset_get_description(row));
			new_path = radio_freq_to_path(freq, tmp);
			g_free(tmp);

			tmp = radio_freq_to_text(freq);
			gtk_clist_set_text(GTK_CLIST(clist), row, column, tmp);
			g_free(tmp);

			gtk_clist_set_row_data_full(GTK_CLIST(clist), row,
						    new_path, (GtkDestroyNotify)g_free);
			}
		}
	else if (column == 2)
		{
		if (event->button == 1)
			{
			clist_edit_by_row(GTK_CLIST(clist), row, column,
                                          preset_clist_description_cb, NULL);
			}
		}
}

static void preset_list_apply(GtkWidget *clist)
{
	gint i;
	gint change = FALSE;

	for (i = 0; i < PRESET_LIST_SIZE; i++)
		{
		const gchar *path;

		path = gtk_clist_get_row_data(GTK_CLIST(clist), i);
		if (path)
			{
			if (strcmp(path, "empty") == 0)
				{
				preset_clear(i);
				}
			else
				{
				preset_set_by_path(i, path);
				}
			change = TRUE;
			gtk_clist_set_row_data(GTK_CLIST(clist), i, NULL);
			}
		}

	if (change)
		{
		if (!preset_is_freq(preset, frequency))
			{
			preset = preset_find_freq(frequency);
			display_update_preset(FALSE, preset);
			}
		display_preset_list_update(-1);
		}
}

static void populate_preset_list(GtkWidget *clist)
{
	gchar *buf[4];
	gint i;

	buf[3] = NULL;
	for (i = 0; i < PRESET_LIST_SIZE; i++)
		{
		guint32 freq;
		const gchar *desc;
		gint row;

		freq = preset_get_freq(i);
		desc = preset_get_description(i);

		buf[0] = g_strdup_printf("%d", i + 1);
		if (freq > 0)
			{
			buf[1] = radio_freq_to_text(preset_get_freq(i));
			}
		else
			{
			buf[1] = g_strdup("");
			}
		buf[2] = g_strdup((desc) ? desc : "");

		row = gtk_clist_append(GTK_CLIST(clist), buf);
		gtk_clist_set_row_data(GTK_CLIST(clist), row, NULL);

		g_free(buf[0]);
		g_free(buf[1]);
		g_free(buf[2]);
		}
}

static void mixer_device_id_cb(GtkWidget *w, gpointer data)
{
	gchar *dev_text = data;
	gint d = get_mixer_device_id(dev_text);
	if (d >= 0) c_mixer_device_id = d;
}

static void spin_cb(GtkObject *adj, gpointer data)
{
	gint *c_n = data;

	*c_n = (gint)(GTK_ADJUSTMENT(adj)->value) * 10000;
}

static void skin_item_selected(GtkWidget *clist, gint row, gint column,
		GdkEventButton *bevent, gpointer data)
{
	gchar *text = gtk_clist_get_row_data(GTK_CLIST(clist), row);
	if (text) gtk_entry_set_text(GTK_ENTRY(default_skin_entry),text);
}

static gint sort_skin_list_cb(void *a, void *b)
{
	const gchar *fa = filename_from_path((gchar *)a);
	const gchar *fb = filename_from_path((gchar *)b);
	if (!fa || !fb) return 0;
	return strcmp(fa, fb);
}

static GList *skin_list_add_dir(GList *list, const gchar *skinpath)
{
	GList *dirs;

	if (!path_list(skinpath, NULL, &dirs)) return list;

	return g_list_concat(list, dirs);
}

static void populate_skin_list(GtkWidget *clist)
{
	gchar *skinpath;
	GList *skin_list = NULL;
	GList *work;
	gchar *buf[2];
	gint row;

	/* add default skin */
	buf[0] = _("None (default)");
	buf[1] = NULL;
	row = gtk_clist_append(GTK_CLIST(clist), buf);
	gtk_clist_set_row_data(GTK_CLIST(clist), row, "");

	/* add home dir skins */
	skinpath = g_strconcat(homedir(), "/", GQRADIO_RC_DIR_SKIN, NULL);
	skin_list = skin_list_add_dir(skin_list, skinpath);
	g_free(skinpath);

	/* add system dir skins */
	skin_list = skin_list_add_dir(skin_list, GQRADIO_SKINDIR);

	skin_list = g_list_sort(skin_list, (GCompareFunc) sort_skin_list_cb);

	work = skin_list;
	while (work)
		{
		gchar *path = work->data;
		gchar *skindata_file;

		work = work->next;

		if (strcmp(path, GQRADIO_SKINDIR "/default") == 0)
			{
			g_free(path);
			continue;
			}

		skindata_file = g_strconcat(path, "/skindata", NULL);
		if (isfile(skindata_file))
			{
			buf[0] = (gchar *)filename_from_path(path);
			row = gtk_clist_append(GTK_CLIST(clist),buf);
			gtk_clist_set_row_data_full(GTK_CLIST(clist), row,
					path, (GtkDestroyNotify) g_free);
			}
		g_free(skindata_file);
		}

	g_list_free(skin_list);
}

void show_config_window(gint start_tab)
{
	if (configwindow)
		{
		gdk_window_raise(configwindow->window);
		return;
		}

	create_config_window(start_tab);
	gtk_widget_show_all(configwindow);
}

static void hide_config_window(void)
{
	gtk_widget_destroy(configwindow);
	configwindow = NULL;
}

static void apply_config_window(void)
{
	const gchar *tmp;
	gchar *buf = NULL;
	gint skin_changed = FALSE;
	gint need_init = FALSE;

	if (slik_double_size != c_slik_double_size) skin_changed = TRUE;

	if (slik_colorshift_on != c_slik_colorshift_on)
		{
		slik_colorshift_on = c_slik_colorshift_on;
		skin_changed = TRUE;
		}

	if (slik_colorshift_r != c_slik_colorshift_r ||
	    slik_colorshift_g != c_slik_colorshift_g ||
	    slik_colorshift_b != c_slik_colorshift_b ||
	    slik_colorshift_a != c_slik_colorshift_a )
		{
		slik_colorshift_r = c_slik_colorshift_r;
		slik_colorshift_g = c_slik_colorshift_g;
		slik_colorshift_b = c_slik_colorshift_b;
		slik_colorshift_a = c_slik_colorshift_a;
		if (slik_colorshift_on) skin_changed = TRUE;
		}

	if (slik_transparency_force != c_slik_transparency_force)
		{
		slik_transparency_force = c_slik_transparency_force;
		skin_changed = TRUE;
		}

	if (slik_transparency_force_a != c_slik_transparency_force_a)
		{
		slik_transparency_force_a = c_slik_transparency_force_a;
		if (slik_transparency_force) skin_changed = TRUE;
		}

	slik_double_size = c_slik_double_size;

	buf = remove_trailing_slash(gtk_entry_get_text(GTK_ENTRY(default_skin_entry)));
	if ( (buf && main_window->skin_path && strcmp(buf, main_window->skin_path) != 0 && strlen(buf) > 0) ||
		(!main_window->skin_path && buf && strlen(buf) > 0) )
		{
		/* change to the new skin */
		ui_skin_load(main_window, buf, NULL);
		tab_completion_append_to_history(default_skin_entry, buf);
		}
	else if (main_window->skin_path && buf && strlen(buf) == 0)
		{
		ui_skin_load(main_window, NULL, NULL);
		}
	else if (skin_changed)
		{
		ui_skin_load(main_window, main_window->skin_path, main_window->skin_mode_key);
		}
	g_free(buf);

	preset_list_apply(preset_clist);

	tmp = gtk_entry_get_text(GTK_ENTRY(device_entry));
	if (( (strlen(tmp) > 0) != (radio_custom_device != NULL) ) ||
	    (radio_custom_device && strcmp(tmp, radio_custom_device) != 0) )
		{
		g_free(radio_custom_device);
		if (strlen(tmp) > 0)
			{
			radio_custom_device = g_strdup(tmp);
			}
		else
			{
			radio_custom_device = NULL;
			}
		need_init = TRUE;
		}

	freq_step = c_freq_step;

	if (mixer_device_id != c_mixer_device_id)
		set_mixer_device(c_mixer_device_id);
	mixer_device_id = c_mixer_device_id;

	if (mixer_command)
		{
		g_free(mixer_command);
		mixer_command = NULL;
		}
	buf = gtk_entry_get_text(GTK_ENTRY(mixer_entry));
	if (buf && strlen(buf) > 0) mixer_command = g_strdup(buf);

	slik_smart_placement = c_slik_smart_placement;
	slik_remember_position = c_slik_remember_position;
	wm_decorations = c_wm_decorations;

	if (description_scrolls_always != c_description_scrolls_always)
		{
		description_scrolls_always = c_description_scrolls_always;
		display_set_description(NULL);
		}

	skinned_menus_enable = c_skinned_menus_enable;

	gdk_window_set_decorations (main_window->window->window, wm_decorations);

	radio_limit_by_card_set(c_use_reported_freq_limits);

	if (need_init)
		{
		radio_shutdown();
		radio_set_device(radio_custom_device);
		if (radio_test(TRUE))
			{
			radio_startup(frequency);
			if (main_window->skin_mode_key && strcmp(main_window->skin_mode_key, "skindata_error") == 0)
				{	
				ui_skin_mode_set(main_window, NULL);
				}
			}
		else
			{
			buf = g_strdup_printf(_("Failed to start %s"), (radio_custom_device) ? radio_custom_device : RADIO_DEVICE);
			warning_dialog(_("Radio initialization error"), buf);
			g_free(buf);
			}
		}
}

static void ok_config_window(void)
{
	apply_config_window();
	hide_config_window();
}

static void create_config_window(gint start_tab)
{
	GtkWidget *configvbox;
	GtkWidget *configtabs;
	GtkWidget *button;
	GtkWidget *hbox;
	GtkWidget *vbox;
	GtkWidget *vbox1;
	GtkWidget *label;
	GtkWidget *frame;
	GtkWidget *scrolled;
	GtkWidget *skin_clist;
	GtkWidget *sep;

        GtkWidget *omenu;
        GtkWidget *menu;
        GtkWidget *item;
	GList *devices;
	GList *list;
	gint lh, cnt;
        GtkWidget *hscale;

	GtkWidget *tabcomp;

	GtkObject *adj;
	GtkWidget *spin;


	gchar *skin_title [] = { _("Available skins:"), };
	gchar *skin_label_text;

	gchar *preset_title [] = { _("#"), _("Frequency"), _("Description"), };

	configwindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_wmclass(GTK_WINDOW(configwindow), "preferences", "GQradio");
	gtk_signal_connect (GTK_OBJECT (configwindow), "delete_event",(GtkSignalFunc) hide_config_window, NULL);
	gtk_window_set_policy (GTK_WINDOW (configwindow), FALSE, FALSE, TRUE);
	gtk_window_set_title (GTK_WINDOW (configwindow), _("GQradio preferences"));
	gtk_container_border_width (GTK_CONTAINER (configwindow), 5);

	configvbox = gtk_vbox_new(FALSE, 5);
	gtk_container_add(GTK_CONTAINER(configwindow),configvbox);
	gtk_widget_show(configvbox);

	hbox = gtk_hbutton_box_new();
        gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbox), 0);
        gtk_button_box_set_child_size(GTK_BUTTON_BOX(hbox), 95, 40);
	gtk_box_pack_end(GTK_BOX(configvbox), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	button = gtk_button_new_with_label(_("Ok"));
	gtk_signal_connect(GTK_OBJECT (button), "clicked",(GtkSignalFunc) ok_config_window, NULL);
	gtk_container_add(GTK_CONTAINER(hbox), button);
	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
	gtk_widget_grab_default(button);
	gtk_widget_show(button);

	button = gtk_button_new_with_label(_("Apply"));
	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) apply_config_window, NULL);
	gtk_container_add(GTK_CONTAINER(hbox), button);
	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
	gtk_widget_show(button);

	button = gtk_button_new_with_label(_("Cancel"));
	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) hide_config_window, NULL);
	gtk_container_add(GTK_CONTAINER(hbox), button);
	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
	gtk_widget_show(button);

	configtabs = gtk_notebook_new();
	gtk_notebook_set_tab_pos (GTK_NOTEBOOK(configtabs), GTK_POS_TOP);
	gtk_box_pack_start (GTK_BOX(configvbox), configtabs, TRUE, TRUE, 0);

	/* general tab */

	frame = gtk_frame_new(NULL);
	gtk_container_border_width (GTK_CONTAINER (frame), 5);
	gtk_widget_show(frame);
	label = gtk_label_new(_("General"));
	gtk_notebook_append_page (GTK_NOTEBOOK (configtabs), frame, label);

	vbox = gtk_vbox_new (FALSE, 0);
	gtk_container_border_width (GTK_CONTAINER (vbox), 5);
	gtk_container_add (GTK_CONTAINER(frame),vbox);
	gtk_widget_show(vbox);
	
	frame = gtk_frame_new(_("Window"));
	gtk_container_border_width (GTK_CONTAINER (frame), 5);
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
	gtk_widget_show(frame);

	vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER(frame),vbox1);
	gtk_widget_show(vbox1);

	button = checkbox_new(_("Smart window placement"), slik_smart_placement, &c_slik_smart_placement);
	gtk_box_pack_start(GTK_BOX(vbox1), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	button = checkbox_new(_("Remember window position"), slik_remember_position, &c_slik_remember_position);
	gtk_box_pack_start(GTK_BOX(vbox1), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	button = checkbox_new(_("Enable window manager decorations"), wm_decorations, &c_wm_decorations);
	gtk_box_pack_start(GTK_BOX(vbox1), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	button = checkbox_new(_("Double sized window"), slik_double_size, &c_slik_double_size);
	gtk_box_pack_start(GTK_BOX(vbox1), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	button = checkbox_new(_("Always scroll description"), description_scrolls_always, &c_description_scrolls_always);
	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	frame = gtk_frame_new(_("Mixer"));
	gtk_container_border_width (GTK_CONTAINER (frame), 5);
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
	gtk_widget_show(frame);

	vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER(frame),vbox1);
	gtk_widget_show(vbox1);

	hbox = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	label = gtk_label_new(_("External Mixer:"));
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_widget_show (label);

	gtk_widget_realize(configwindow);

	tabcomp = tab_completion_new(&mixer_entry, mixer_command, NULL, NULL);
	gtk_box_pack_start(GTK_BOX(hbox), tabcomp,TRUE,TRUE,5);
	gtk_widget_show(tabcomp);

	c_mixer_device_id = mixer_device_id;
	hbox = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	label = gtk_label_new(_("Volume controls device:"));
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_widget_show (label);

        omenu = gtk_option_menu_new ();
        menu = gtk_menu_new ();

	devices = get_mixer_device_list();

	list = devices;

	lh = 0;
	cnt = 0;
	while (list)
		{
		gchar *dev_text = list->data;
	        item = gtk_menu_item_new_with_label(dev_text);
	        gtk_signal_connect (GTK_OBJECT (item), "activate", (GtkSignalFunc) mixer_device_id_cb, dev_text);
        	gtk_menu_append (GTK_MENU (menu), item);
		gtk_widget_show(item);
		if (get_mixer_device_id(dev_text) == mixer_device_id) lh = cnt;
		cnt++;
		list = list->next;
		}

	g_list_free(devices);

	gtk_option_menu_set_menu (GTK_OPTION_MENU (omenu), menu);
        gtk_option_menu_set_history (GTK_OPTION_MENU (omenu), lh);
        gtk_box_pack_start( GTK_BOX(hbox), omenu, TRUE, TRUE, 5);
        gtk_widget_show (omenu);

	/* tuner tab */

	frame = gtk_frame_new(NULL);
	gtk_container_border_width (GTK_CONTAINER (frame), 5);
	gtk_widget_show(frame);
	label = gtk_label_new(_("Tuner"));
	gtk_notebook_append_page (GTK_NOTEBOOK (configtabs), frame, label);

	vbox = gtk_vbox_new (FALSE, 0);
	gtk_container_border_width (GTK_CONTAINER (vbox), 5);
	gtk_container_add (GTK_CONTAINER(frame),vbox);
	gtk_widget_show(vbox);
	
	frame = gtk_frame_new(_("Device"));
	gtk_container_border_width (GTK_CONTAINER (frame), 5);
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
	gtk_widget_show(frame);

	vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER(frame),vbox1);
	gtk_widget_show(vbox1);

	device_entry = gtk_entry_new_with_max_length(255);
	if (radio_custom_device) gtk_entry_set_text(GTK_ENTRY(device_entry), radio_custom_device);
	gtk_box_pack_start(GTK_BOX(vbox1), device_entry, FALSE, FALSE, 0);
	gtk_widget_show(device_entry);

	button = checkbox_new(_("Use frequency limits reported by driver"), radio_limit_by_card_get(), &c_use_reported_freq_limits);
	gtk_box_pack_start(GTK_BOX(vbox1), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	hbox = gtk_hbox_new (FALSE, 5);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	label = gtk_label_new(_("Frequency step (x10Khz):"));
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_widget_show(label);

	c_freq_step = freq_step;
	adj = gtk_adjustment_new((gfloat)freq_step / 10000.0, 1.0, 100.0, 1.0, 1.0, 1.0);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1.0, 0);
	gtk_signal_connect(adj, "value_changed",
			   GTK_SIGNAL_FUNC(spin_cb), &c_freq_step);
	gtk_box_pack_start(GTK_BOX(hbox), spin, FALSE, FALSE, 0);
	gtk_widget_show(spin);

	/* preset tab */

	frame = gtk_frame_new(NULL);
	gtk_container_border_width (GTK_CONTAINER (frame), 5);
	gtk_widget_show(frame);
	label = gtk_label_new(_("Presets"));
	gtk_notebook_append_page (GTK_NOTEBOOK (configtabs), frame, label);

	vbox = gtk_vbox_new (FALSE, 0);
	gtk_container_border_width (GTK_CONTAINER (vbox), 5);
	gtk_container_add (GTK_CONTAINER(frame),vbox);
	gtk_widget_show(vbox);

	scrolled = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
				GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
	gtk_box_pack_start(GTK_BOX(vbox), scrolled, TRUE, TRUE, 0);
	gtk_widget_show(scrolled);

	/* list */
	preset_clist=gtk_clist_new_with_titles (3, preset_title);
	gtk_clist_column_titles_passive (GTK_CLIST(preset_clist));
	gtk_clist_set_column_auto_resize(GTK_CLIST(preset_clist), 0, TRUE);
	gtk_signal_connect(GTK_OBJECT(preset_clist), "button_press_event",
                           GTK_SIGNAL_FUNC(preset_clist_click_cb), NULL);
	gtk_container_add(GTK_CONTAINER(scrolled), preset_clist);
	gtk_widget_show(preset_clist);

	populate_preset_list(preset_clist);

	/* skin tab */

	frame = gtk_frame_new(NULL);
	gtk_container_border_width (GTK_CONTAINER (frame), 5);
	gtk_widget_show(frame);
	label = gtk_label_new(_("Skins"));
	gtk_notebook_append_page (GTK_NOTEBOOK (configtabs), frame, label);

	vbox = gtk_vbox_new (FALSE, 0);
	gtk_container_border_width (GTK_CONTAINER (vbox), 5);
	gtk_container_add (GTK_CONTAINER(frame),vbox);
	gtk_widget_show(vbox);

	hbox = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	label = gtk_label_new(_("Skin:"));
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
	gtk_widget_show (label);

	tabcomp = tab_completion_new_with_history(&default_skin_entry,
						  main_window->skin_path ? main_window->skin_path : "",
						  "select_skin", 16, NULL, NULL);
	gtk_box_pack_start(GTK_BOX(hbox), tabcomp, TRUE, TRUE, 0);
	gtk_widget_show(tabcomp);

	scrolled = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
				GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
	gtk_box_pack_start(GTK_BOX(vbox), scrolled, TRUE, TRUE, 0);
	gtk_widget_show(scrolled);

	/* skin list */
	skin_clist=gtk_clist_new_with_titles (1, skin_title);
	gtk_clist_column_titles_passive (GTK_CLIST (skin_clist)); 
	gtk_signal_connect (GTK_OBJECT (skin_clist), "select_row",(GtkSignalFunc) skin_item_selected, NULL);
	gtk_container_add (GTK_CONTAINER (scrolled), skin_clist);
	gtk_widget_show (skin_clist);

	populate_skin_list(skin_clist);

	skin_label_text = g_strconcat(homedir(), "/", GQRADIO_RC_DIR_SKIN, NULL);

	label = gtk_label_new(skin_label_text);
	gtk_box_pack_end(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	gtk_widget_show (label);

	g_free(skin_label_text);

	frame = gtk_frame_new(_("Skin Adjustment"));
	gtk_container_border_width (GTK_CONTAINER (frame), 5);
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
	gtk_widget_show(frame);

	vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER(frame), vbox1);
	gtk_widget_show(vbox1);

	hbox = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	button = checkbox_new(_("Color shift (RGB)"), slik_colorshift_on, &c_slik_colorshift_on);
	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	hscale = hscale_new(slik_colorshift_r, &c_slik_colorshift_r);
        gtk_box_pack_start(GTK_BOX(hbox), hscale, TRUE, TRUE, 0);
        gtk_widget_show(hscale);

	hscale = hscale_new(slik_colorshift_g, &c_slik_colorshift_g);
        gtk_box_pack_start(GTK_BOX(hbox), hscale, TRUE, TRUE, 0);
        gtk_widget_show(hscale);

	hscale = hscale_new(slik_colorshift_b, &c_slik_colorshift_b);
        gtk_box_pack_start(GTK_BOX(hbox), hscale, TRUE, TRUE, 0);
        gtk_widget_show(hscale);
	
	label = gtk_label_new(_("Percent:"));
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_widget_show(label);

	hscale = hscale_new(slik_colorshift_a, &c_slik_colorshift_a);
        gtk_box_pack_start(GTK_BOX(hbox), hscale, TRUE, TRUE, 0);
        gtk_widget_show(hscale);

	hbox = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	button = checkbox_new(_("Force transparency"), slik_transparency_force, &c_slik_transparency_force);
	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	label = gtk_label_new(_("Percent:"));
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_widget_show(label);

	hscale = hscale_new(slik_transparency_force_a, &c_slik_transparency_force_a);
        gtk_box_pack_start(GTK_BOX(hbox), hscale, TRUE, TRUE, 0);
        gtk_widget_show(hscale);

	button = checkbox_new(_("Enable menu skins"), skinned_menus_enable, &c_skinned_menus_enable);
	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	sep = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(vbox), sep, FALSE, FALSE, 0);
	gtk_widget_show(sep);

	gtk_notebook_set_page(GTK_NOTEBOOK(configtabs), start_tab);

	gtk_widget_show(configtabs);
}


